# -*- coding: utf-8 -*-
"""
/***************************************************************************
 vector
                                 A QGIS plugin
 vector
                              -------------------
        begin                : 2016-04-04
        git sha              : $Format:%H$
        copyright            : (C) 2016 by vector
        email                : vector
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt4.QtCore import QSettings, QTranslator, qVersion, QCoreApplication
from PyQt4.QtGui import QAction, QIcon
from PyQt4 import QtGui
from qgis.core import QgsFeature,QgsVectorLayer,QGis
# Initialize Qt resources from file resources.py
import resources
# Import the code for the dialog
from vector_dialog import vectorDialog
import os.path


class vector:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'vector_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

        # Create the dialog (after translation) and keep reference
        self.dlg = vectorDialog()

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&vector')
        # TODO: We are going to let the user set this up in a future iteration
        self.toolbar = self.iface.addToolBar(u'vector')
        self.toolbar.setObjectName(u'vector')

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('vector', message)


    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            self.toolbar.addAction(action)

        if add_to_menu:
            self.iface.addPluginToVectorMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/vector/icon.png'
        self.add_action(
            icon_path,
            text=self.tr(u'vector'),
            callback=self.run,
            parent=self.iface.mainWindow())


    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginVectorMenu(
                self.tr(u'&vector'),
                action)
            self.iface.removeToolBarIcon(action)
        # remove the toolbar
        del self.toolbar


    def run(self):
        """Run method that performs all the real work"""
        # show the dialog
        self.dlg.show()
        self.initdlg()
        # Run the dialog event loop
        result = self.dlg.exec_()
        # See if OK was pressed
        if result:
            # Do something useful here - delete the line containing pass and
            # substitute with your code.
            pass

    def initdlg(self):
        layer = self.iface.activeLayer()
        if not layer:
            QtGui.QMessageBox.about(self.dlg,'tips',"no activity layer!")
            return
        iter = layer.getFeatures()
        target_type = 'unknow'
        target_count = 0
        point_count = 0
        for feature in iter:
            geom = feature.geometry()
            print "Feature ID %d: " % feature.id()
            target_count+=1
            # show some information about the feature
            if geom.type() == QGis.Point:
                target_type = 'poi'
                x = geom.asPoint()
                point_count += 1
                print "Point: " + str(x)
            elif geom.type() == QGis.Line:
                target_type = 'line'
                x = geom.asPolyline()
                point_count += len(x)
                print "Line: %d points" % len(x)
            elif geom.type() == QGis.Polygon:
                target_type = 'polygon'
                #首先转化为geomcollection
                #QList<QgsGeometry*>QgsGeometry::asGeometryCollection() const
                x = geom.asGeometryCollection()
                numPts = 0
                #第二步,遍历collection,得到一个简单多边形
                for geom_polygon in x:
                    #第三步,将geometry转化为polygon,得到一个简单多边形,遍历简单多边形内部环
                    geompolygon = geom_polygon.asPolygon()
                    for ring in geompolygon:
                        numPts += len(ring)
                point_count += numPts
            else:
                target_type = 'unknown'
                print "Unknown"
                break
        self.dlg.targetcount.setValue(target_count);
        self.dlg.pointcount.setValue(point_count);
        self.dlg.currenttarget.addItem(target_type)
        return








